@@ -5,13 +5,14 @@ use Exporter       ();
 use ORLite::Mirror ();
 use Test::More     ();
 use File::Remove   ();
+use File::HomeDir  ();
 use File::Spec::Functions ':ALL';
 
 use vars qw{$VERSION @ISA @EXPORT};
 BEGIN {
-	$VERSION = '1.12';
-	@ISA     = qw{ Exporter };
-	@EXPORT  = qw{ test_db connect_ok create_ok };
+	$VERSION = '1.14';
+	@ISA     = 'Exporter';
+	@EXPORT  = qw{ test_db mirror_db connect_ok create_ok };
 }
 
 
@@ -27,6 +28,17 @@ sub test_db {
 	return $file;
 }
 
+sub mirror_db {
+	my $dir = catdir(
+		File::HomeDir->my_data,
+		($^O eq 'MSWin32' ? 'Perl' : '.perl'),
+		'ORLite-Mirror',
+	);
+	my $file = shift;
+	$file =~ s/::/-/g;
+	return catfile( $dir, "$file.sqlite" );
+}
+
 sub connect_ok {
 	my $dbh = DBI->connect(@_);
 	Test::More::isa_ok( $dbh, 'DBI::db' );
